exec("./sounds.cs");
if(!$Morse::EventInit) {
	exec("./events.cs");
}

if(!$Pref::Server::Morse::Delay) {
	$Pref::Server::Morse::Delay = 120;
	$Pref::Server::Morse::Debug = 0;
}

exec("./prefs.cs");

$Pref::Server::Morse::Delay = mClamp($Pref::Server::Morse::Delay, 80, 300);

function convertCharacterToMorse(%char) {
	if(strLen(%char) > 1 || %char $= "") {
		return "E";
	}

	// imagine if these were all if statements
	switch$(%char) {
		case "A": return ".-";
		case "B": return "-...";
		case "C": return "-.-.";
		case "D": return "-..";
		case "E": return ".";
		case "F": return "..-.";
		case "G": return "--.";
		case "H": return "....";
		case "I": return "..";
		case "J": return ".---";
		case "K": return "-.-";
		case "L": return ".-..";
		case "M": return "--";
		case "N": return "-.";
		case "O": return "---";
		case "P": return ".--.";
		case "Q": return "--.-";
		case "R": return ".-.";
		case "S": return "...";
		case "T": return "-";
		case "U": return "..-";
		case "V": return "...-";
		case "W": return ".--";
		case "X": return "-..-";
		case "Y": return "-.--";
		case "Z": return "--..";
		case "0": return "-----";
		case "1": return ".----";
		case "2": return "..---";
		case "3": return "...--";
		case "4": return "....-";
		case "5": return ".....";
		case "6": return "-....";
		case "7": return "--...";
		case "8": return "---..";
		case "9": return "----.";
		case ".": return ".-.-.-";
		case ",": return "--..--";
		case ":": return "---...";
		case "?": return "..--..";
		case "'": return ".----.";
		case "-": return "-....-";
		case "/": return "-..-.";
		case "(" or ")" or "[" or "]": return "-.--.-";
		case "\"": return ".-..-.";
		case " ": return "/";
		case "@": return ".--.-.";
		case "=": return "-...-";
	}
}

function sterilizeMorseString(%string) {
	return stripChars(strUpr(%string), "!#$%^&*_+\\{}|;<>`~");
}
function convertToMorse(%string) {
	%string = sterilizeMorseString(%string);

	%output = "";
	for(%i=0;%i<strLen(%string);%i++) {
		%char = getSubStr(%string, %i, 1);
		%output = trim(%output SPC convertCharacterToMorse(%char));
	}

	return %output;
}

function playMorseSound(%this, %char) {
	if(!isObject(%this)) {
		return;
	}

	%class = %this.getClassName();
	switch$(%class) {
		case "GameConnection":
			switch$(%char) {
				case ".": %this.play2D(morseDotSound);
				case "-": %this.play2D(morseDashSound);
			}

		case "Player" or "fxDTSBrick":
			switch$(%char) {
				case ".": serverPlay3D(morseDotSound, %this.getPosition());
				case "-": serverPlay3D(morseDashSound, %this.getPosition());
			}
	}
}

function determineMorseDelayOffset(%char) {
	switch$(%char) {
		case ".": return $Pref::Server::Morse::Delay;
		case "-" or "/": return $Pref::Server::Morse::Delay*2;
	}
}

function playMorseSequence(%this, %sequence) {
	if(!isObject(%this)) {
		return;
	}

	if(getWordCount(%sequence) > 0) {
		%this.isPlayingMorse = 1;

		%word = getWord(%sequence, 0);
		if($Pref::Server::Morse::Debug) {
			echo(%word);
		}

		%length = strLen(%word);

		%delay = 0;
		for(%i=0;%i<%length;%i++) {
			%char = getSubStr(%word, %i, 1);

			schedule(%delay, 0, playMorseSound, %this, %char);
			%delay += determineMorseDelayOffset(%char);
		}

		schedule(($Pref::Server::Morse::Delay*4) + %delay, 0, playMorseSequence, %this, getWords(%sequence, 1, getWordCount(%sequence)));
	} else {
		%this.isPlayingMorse = 0;
	}
}

package MorseSupportPackage {
	function onServerDestroyed() {
		deleteVariables("$Morse::*");
		return parent::onServerDestroyed();
	}
};
activatePackage(MorseSupportPackage);

$Morse::Version = "1.0.0-1";
echo("Executed Event_Morse v" @ $Morse::Version);